# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: https://docs.scrapy.org/en/latest/topics/item-pipeline.html


# useful for handling different item types with a single interface
from itemadapter import ItemAdapter
from scrapy.exporters import CsvItemExporter


class ScrapyDemoPipeline:
    def process_item(self, item, spider):
        return item


class SpiderMongoDemoPipeline:
    def open_spider(self, spider):
        from scrapy_demo.scrapy_demo.settings import HOST, PORT, DB, COLLECTION
        import pymongo
        self.mongo_client = pymongo.MongoClient(HOST, PORT)
        self.db = self.mongo_client[DB]
        self.sheet = self.db[COLLECTION]

    def process_item(self, item, spider):
        print('Pipline:', item)
        self.sheet.insert_one(dict(item))
        return item

    def close_spider(self, spider):
        self.mongo_client.close()


class SpiderCsvPipeline(object):
    def open_spider(self, spider):
        from scrapy_demo.scrapy_demo.settings import FILE_PATH
        # 创建csv格式的文件
        self.file = open(FILE_PATH, "wb+")
        # 创建csv文件读写对象，将数据写入到指定的文件中
        self.csv_exporter = CsvItemExporter(self.file)
        # 开始执行item数据读写
        self.csv_exporter.start_exporting()

    def process_item(self, item, spider):
        print("CSV:", item)
        # 将item数据写入到文件中
        self.csv_exporter.export_item(item)
        return item

    def close_spider(self, spider):
        # 结束文件读写操作
        self.csv_exporter.finish_exporting()
        # 关闭文件
        self.file.close()

class SpiderMySQLPipeline:
    def open_spider(self, spider):
        pass

    def process_item(self, item, spider):
        print("MySQL:", item)
        pass

    def close_spider(self, spider):
        pass


from copy import deepcopy

import scrapy

from scrapy_demo.scrapy_demo import settings
from scrapy import Item, Field



def xpath_item(item, datas, response):
    for data in datas:
        if '/' in data:
            ul_li = response.xpath(data)
            for ul in ul_li:
                xpath_item(item, datas[data], ul)
        else:
            item.fields[data] = Field()
            item[data] = response.xpath(datas[data]).extract_first()
    return item


class BaiduSpider(scrapy.Spider):
    name = 'test'
    allowed_domains = settings.ALLOWED_DOMAINS
    start_urls = settings.START_URL

    def parse(self, response):
        item = Item()
        datas = settings.DATAS
        for data in datas:
            if '/' in data:
                ul_li = response.xpath(data)
                for ul in ul_li:
                    xpath_item(item, datas[data], ul)
                    yield item
            else:
                item.fields[data] = Field()
                item[data] = response.xpath(datas[data]).extract_first()




# -*- coding: utf-8 -*-
# @Author : zhy
# @Time   : 2022-05-30
# @File   : setup.py
from setuptools import setup,find_packages

setup(
    name='scrapy_demo',
    version='8.0',
    description='scrapy通用爬虫',
    include_package_data=True,  # 是否允许上传资源文件
    packages=find_packages(),#包的目录
    author='zhy',  # 作者
    author_email='191517137@qq.com',  # 作者邮件
    python_requires='==3.7.5',#设置python版本要求
    install_requires=['scrapy']#安装所需要的库
)



# -*- coding: utf-8 -*-
# @Author : zhy
# @Time   : 2022-05-25
# @File   : run_spider.py
import json
import os

from scrapy import cmdline


from scrapy_demo.scrapy_demo import settings


class Spider(object):
    def __init__(self, file_path):
        self.file_path = file_path

    # pipline
    @staticmethod
    def change_pipline(pipeline_data: dict):
        save_type = pipeline_data.get('type')
        if save_type == 'mongo':
            settings.HOST = pipeline_data.get('host')
            settings.PORT = pipeline_data.get('port')
            settings.DB = pipeline_data.get('db')
            settings.COLLECTION = pipeline_data.get("collection")
            settings.ITEM_PIPELINES = {
                'scrapy_demo.scrapy_demo.pipelines.SpiderMongoDemoPipeline': 300,
            }
        elif save_type == 'csv':
            settings.FILE_PATH = pipeline_data.get('file_path')
            settings.ITEM_PIPELINES = {
                'scrapy_demo.scrapy_demo.pipelines.SpiderCsvPipeline': 300,
            }
        elif save_type == 'mysql':
            pass
        else:
            pass

    # setting
    @staticmethod
    def change_setting(json_data: dict):
        settings.DATAS = json_data.get('datas')
        settings.RETRY_TIMES = json_data.get('RETRY_TIMES')
        settings.DOWNLOAD_DELAY = json_data.get('DOWNLOAD_DELAY')
        headers = json_data.get('DEFAULT_REQUEST_HEADERS')
        if headers == None:
            settings.DOWNLOADER_MIDDLEWARES = {
                'scrapy_demo.scrapy_demo.middlewares.ScrapyDemoDownloaderMiddleware': 543,
            }
        else:
            settings.DEFAULT_REQUEST_HEADERS = json_data.get('DEFAULT_REQUEST_HEADERS')
        settings.ALLOWED_DOMAINS = json_data.get('allowed_domains')
        settings.START_URL = json_data.get('start_url')
        # settings.START_PAGE = json_data.get('start_page')
        # settings.END_PAGE = json_data.get('end_page')

    def run(self):
        json_data = json.load(open(self.file_path, 'r', encoding="utf-8"))
        self.change_pipline(pipeline_data=json_data.get('save_pipeline'))
        self.change_setting(json_data=json_data)
        import os
        os.chdir(os.path.dirname(os.path.abspath(__file__)))  # 切换运行路径，解决找不到scrapy.cfg问题
        # 运行爬虫
        cmdline.execute('scrapy crawl test'.split())
        pass


if __name__ == '__main__':
    file_path = r'C:\Users\17580\Desktop\scrapy-redis分享\demo.json'
    spider = Spider(file_path=file_path)
    spider.run()
